@@ -15,7 +15,7 @@ when 'Linux'
 when 'AIX'
   packagename = 'bos.net.tcp.client'
 when 'Solaris'
-  case fact('operatingsystemrelease')
+  case fact('kernelrelease')
   when '5.10'
     packagename = ['SUNWntpr','SUNWntpu']
   when '5.11'
@@ -29,6 +29,14 @@ else
   end
 end
 
+if (fact('osfamily') == 'RedHat')
+  keysfile = '/etc/ntp/keys'
+elsif (fact('osfamily') == 'Solaris')
+  keysfile = '/etc/inet/ntp.keys'
+else
+  keysfile = '/etc/ntp.keys'
+end
+
 if (fact('osfamily') == 'Solaris')
   config = '/etc/inet/ntp.conf'
 else
@@ -100,25 +108,27 @@ describe "ntp class:", :unless => UNSUPPORTED_PLATFORMS.include?(fact('osfamily'
       pp = <<-EOS
       class { 'ntp':
         keys_enable     => true,
-        keys_file       => '/etc/ntp/keys',
-        keys_controlkey => '/etc/ntp/controlkey',
+        keys_controlkey => '15',
         keys_requestkey => '1',
         keys_trusted    => [ '1', '2' ],
+        keys            => [ '1 M AAAABBBB' ],
       }
       EOS
-      # Rely on a shell command instead of a file{} here to avoid loops
-      # within puppet when it tries to manage /etc/ntp/keys before /etc/ntp.
-      shell("mkdir -p /etc/ntp && echo '1 M AAAABBBB' >> /etc/ntp/keys")
       apply_manifest(pp, :catch_failures => true)
     end
 
     describe file("#{config}") do
       it { should be_file }
-      its(:content) { should match 'keys /etc/ntp/keys' }
-      its(:content) { should match 'controlkey /etc/ntp/controlkey' }
+      its(:content) { should match "keys #{keysfile}" }
+      its(:content) { should match 'controlkey 15' }
       its(:content) { should match 'requestkey 1' }
       its(:content) { should match 'trustedkey 1 2' }
     end
+
+    describe file(keysfile) do
+      it { should be_file }
+      its(:content) { should match '1 M AAAABBBB' }
+    end
   end
 
   describe 'package' do
