@@ -3,23 +3,20 @@ require 'spec_helper'
 describe 'ntp' do
   let(:facts) {{ :is_virtual => 'false' }}
 
-  ['Debian', 'RedHat', 'Fedora', 'Suse', 'FreeBSD', 'Archlinux', 'Gentoo', 'Gentoo (Facter < 1.7)'].each do |system|
+  ['Debian', 'RedHat', 'Fedora', 'Suse11', 'Suse12', 'FreeBSD', 'Archlinux', 'Gentoo', 'Gentoo (Facter < 1.7)'].each do |system|
     context "when on system #{system}" do
-      if system == 'Gentoo (Facter < 1.7)'
-        let :facts do
+      let :facts do
+        case system
+        when 'Gentoo (Facter < 1.7)'
           super().merge({ :osfamily => 'Linux', :operatingsystem => 'Gentoo' })
-        end
-      elsif system == 'Suse'
-        let :facts do
-          super().merge({ :osfamily => system,:operatingsystem => 'SLES',:operatingsystemmajrelease => '11' })
-        end
-      elsif system == 'Fedora'
-        let :facts do
-          super().merge({ :osfamily => 'RedHat', :operatingsystem => system ,:operatingsystemmajrelease => '22' })
-        end
-      else
-        let :facts do
-          super().merge({ :osfamily => system })
+        when 'Suse11'
+          super().merge({ :osfamily => 'Suse', :operatingsystem => 'SLES', :operatingsystemmajrelease => '11' })
+        when 'Suse12'
+          super().merge({ :osfamily => 'Suse', :operatingsystem => 'SLES', :operatingsystemmajrelease => '12' })
+        when 'Fedora'
+          super().merge({ :osfamily => 'RedHat', :operatingsystem => system, :operatingsystemmajrelease => '22' })
+        else
+          super().merge({ :osfamily => system, :operatingsystem => system})
         end
       end
 
@@ -32,6 +29,10 @@ describe 'ntp' do
         it { should contain_file('/etc/ntp.conf').with_group('0') }
         it { should contain_file('/etc/ntp.conf').with_mode('0644') }
 
+        if system == 'Suse12'
+          it { should contain_file('/var/run/ntp/servers-netconfig').with_ensure_absent }
+        end
+
         describe 'allows template to be overridden' do
           let(:params) {{ :config_template => 'my_ntp/ntp.conf.erb' }}
           it { should contain_file('/etc/ntp.conf').with({
@@ -43,15 +44,11 @@ describe 'ntp' do
           context "when enabled" do
             let(:params) {{
               :keys_enable     => true,
-              :keys_file       => '/etc/ntp/ntp.keys',
               :keys_trusted    => ['1', '2', '3'],
               :keys_controlkey => '2',
               :keys_requestkey => '3',
             }}
 
-            it { should contain_file('/etc/ntp').with({
-              'ensure'  => 'directory'})
-            }
             it { should contain_file('/etc/ntp.conf').with({
               'content' => /trustedkey 1 2 3/})
             }
@@ -67,15 +64,11 @@ describe 'ntp' do
         context "when disabled" do
           let(:params) {{
             :keys_enable     => false,
-            :keys_file       => '/etc/ntp/ntp.keys',
             :keys_trusted    => ['1', '2', '3'],
             :keys_controlkey => '2',
             :keys_requestkey => '3',
           }}
 
-          it { should_not contain_file('/etc/ntp').with({
-            'ensure'  => 'directory'})
-          }
           it { should_not contain_file('/etc/ntp.conf').with({
             'content' => /trustedkey 1 2 3/})
           }
@@ -131,6 +124,30 @@ describe 'ntp' do
             }
           end
         end
+
+        describe 'specified ignore interfaces' do
+          context "when set" do
+            let(:params) {{
+              :interfaces => ['a.b.c.d'],
+              :interfaces_ignore => ['wildcard', 'ipv6']
+            }}
+
+            it { should contain_file('/etc/ntp.conf').with({
+              'content' => /interface ignore wildcard\ninterface ignore ipv6\ninterface listen a.b.c.d/})
+            }
+          end
+          context "when not set" do
+            let(:params) {{
+              :interfaces   => ['127.0.0.1'],
+              :servers      => ['a', 'b', 'c', 'd'],
+            }}
+
+            it { should contain_file('/etc/ntp.conf').with({
+              'content' => /interface ignore wildcard\ninterface listen 127.0.0.1/})
+            }
+          end
+        end
+
         describe 'with parameter disable_auth' do
           context 'when set to true' do
             let(:params) {{
@@ -150,7 +167,58 @@ describe 'ntp' do
 
             it 'should not contain disable auth setting' do
               should_not contain_file('/etc/ntp.conf').with({
-              'content' => /^disable auth\n/,
+                'content' => /^disable auth\n/,
+              })
+            end
+          end
+        end
+
+        describe 'with parameter disable_dhclient' do
+          context 'when set to true' do
+            let(:params) {{
+              :disable_dhclient => true,
+            }}
+
+            it 'should contain disable ntp-servers setting' do
+              should contain_augeas('disable ntp-servers in dhclient.conf')
+            end
+            it 'should contain dhcp file' do
+              should contain_file('/var/lib/ntp/ntp.conf.dhcp').with_ensure('absent')
+            end
+          end
+          context 'when set to false' do
+            let(:params) {{
+              :disable_dhclient => false,
+            }}
+
+            it 'should not contain disable ntp-servers setting' do
+              should_not contain_augeas('disable ntp-servers in dhclient.conf')
+            end
+            it 'should not contain dhcp file' do
+              should_not contain_file('/var/lib/ntp/ntp.conf.dhcp').with_ensure('absent')
+            end
+          end
+        end
+        describe 'with parameter disable_kernel' do
+          context 'when set to true' do
+            let(:params) {{
+              :disable_kernel => true,
+            }}
+
+            it 'should contain disable kernel setting' do
+              should contain_file('/etc/ntp.conf').with({
+              'content' => /^disable kernel\n/,
+              })
+            end
+          end
+          context 'when set to false' do
+            let(:params) {{
+              :disable_kernel => false,
+            }}
+
+            it 'should not contain disable kernel setting' do
+              should_not contain_file('/etc/ntp.conf').with({
+              'content' => /^disable kernel\n/,
               })
             end
           end
@@ -162,7 +230,7 @@ describe 'ntp' do
 
             it 'should contain disable monitor setting' do
               should contain_file('/etc/ntp.conf').with({
-              'content' => /^disable monitor\n/,
+                'content' => /^disable monitor\n/,
               })
             end
           end
@@ -173,7 +241,7 @@ describe 'ntp' do
 
             it 'should contain disable monitor setting' do
               should contain_file('/etc/ntp.conf').with({
-              'content' => /^disable monitor\n/,
+                'content' => /^disable monitor\n/,
               })
             end
           end
@@ -184,7 +252,7 @@ describe 'ntp' do
 
             it 'should not contain disable monitor setting' do
               should_not contain_file('/etc/ntp.conf').with({
-              'content' => /^disable monitor\n/,
+                'content' => /^disable monitor\n/,
               })
             end
           end
@@ -197,7 +265,7 @@ describe 'ntp' do
 
             it 'should contain broadcastclient setting' do
               should contain_file('/etc/ntp.conf').with({
-              'content' => /^broadcastclient\n/,
+                'content' => /^broadcastclient\n/,
               })
             end
           end
@@ -208,10 +276,36 @@ describe 'ntp' do
 
             it 'should not contain broadcastclient setting' do
               should_not contain_file('/etc/ntp.conf').with({
-              'content' => /^broadcastclient\n/,
+                'content' => /^broadcastclient\n/,
               })
             end
           end
+          context 'when setting custom config_dir' do
+            let(:params) {{
+              :keys_enable => true,
+              :config_dir  => '/tmp/foo',
+              :keys_file   => '/tmp/foo/ntp.keys',
+            }}
+
+            it 'should contain custom config directory' do
+              should contain_file('/tmp/foo').with(
+                'ensure'  => 'directory',
+                'owner'   => '0',
+                'group'   => '0',
+                'mode'    => '0775',
+                'recurse' => 'false'
+              )
+            end
+          end
+          context 'when manually setting conf file mode to 0777' do
+            let(:params) {{
+              :config_file_mode => '0777',
+            }}
+
+            it 'should contain file mode of 0777' do
+              should contain_file('/etc/ntp.conf').with_mode('0777')
+            end
+          end
         end
 
         describe "ntp::install on #{system}" do
@@ -506,6 +600,86 @@ describe 'ntp' do
           end
         end
 
+        describe 'with parameter ntpsigndsocket' do
+          context 'when set to true' do
+            let(:params) {{
+                :servers => ['a', 'b', 'c', 'd'],
+                :ntpsigndsocket => '/usr/local/samba/var/lib/ntp_signd',
+            }}
+
+            it 'should contain ntpsigndsocket setting' do
+              should contain_file('/etc/ntp.conf').with({
+                'content' => %r(^ntpsigndsocket /usr/local/samba/var/lib/ntp_signd\n),
+              })
+            end
+          end
+
+          context 'when set to false' do
+            let(:params) {{
+                :servers => ['a', 'b', 'c', 'd'],
+            }}
+
+            it 'should not contain a ntpsigndsocket line' do
+              should_not contain_file('/etc/ntp.conf').with({
+                'content' => /ntpsigndsocket /,
+              })
+            end
+          end
+        end
+
+        describe 'with parameter authprov' do
+          context 'when set to true' do
+            let(:params) {{
+                :servers => ['a', 'b', 'c', 'd'],
+                :authprov => '/opt/novell/xad/lib64/libw32time.so 131072:4294967295 global',
+            }}
+
+            it 'should contain authprov setting' do
+              should contain_file('/etc/ntp.conf').with({
+                'content' => %r(^authprov /opt/novell/xad/lib64/libw32time.so 131072:4294967295 global\n),
+              })
+            end
+          end
+
+          context 'when set to false' do
+            let(:params) {{
+                :servers => ['a', 'b', 'c', 'd'],
+            }}
+
+            it 'should not contain a authprov line' do
+              should_not contain_file('/etc/ntp.conf').with({
+                'content' => /authprov /,
+              })
+            end
+          end
+        end
+
+        describe 'with parameter tos' do
+          context 'when set to true' do
+            let(:params) {{
+              :tos     => true,
+            }}
+
+            it 'should contain tos setting' do
+              should contain_file('/etc/ntp.conf').with({
+              'content' => /^tos/,
+              })
+            end
+          end
+
+          context 'when set to false' do
+            let(:params) {{
+              :tos     => false,
+            }}
+
+            it 'should not contain tos setting' do
+              should_not contain_file('/etc/ntp.conf').with({
+                'content' => /^tos/,
+              })
+            end
+          end
+        end
+
         describe 'peers' do
           context 'when empty' do
             let(:params) do
@@ -539,7 +713,7 @@ describe 'ntp' do
       describe "for operating system Gentoo (Facter < 1.7)" do
         let :facts do
           super().merge({ :operatingsystem => 'Gentoo',
-                       :osfamily        => 'Linux' })
+                          :osfamily        => 'Linux' })
         end
 
         it 'uses the NTP pool servers by default' do
@@ -551,7 +725,8 @@ describe 'ntp' do
 
       describe "on osfamily Gentoo" do
         let :facts do
-          super().merge({ :osfamily => 'Gentoo' })
+          super().merge({ :osfamily        => 'Gentoo',
+                          :operatingsystem => 'Gentoo' })
         end
 
         it 'uses the NTP pool servers by default' do
@@ -563,7 +738,8 @@ describe 'ntp' do
 
       describe "on osfamily Debian" do
         let :facts do
-          super().merge({ :osfamily => 'debian' })
+          super().merge({ :osfamily        => 'debian',
+                          :operatingsystem => 'debian' })
         end
 
         it 'uses the debian ntp servers by default' do
@@ -575,7 +751,8 @@ describe 'ntp' do
 
       describe "on osfamily RedHat" do
         let :facts do
-          super().merge({ :osfamily => 'RedHat' })
+          super().merge({ :osfamily        => 'RedHat',
+                          :operatingsystem => 'RedHat' })
         end
 
         it 'uses the redhat ntp servers by default' do
@@ -599,7 +776,8 @@ describe 'ntp' do
 
       describe "on osfamily FreeBSD" do
         let :facts do
-          super().merge({ :osfamily => 'FreeBSD' })
+          super().merge({ :osfamily        => 'FreeBSD',
+                          :operatingsystem => 'FreeBSD' })
         end
 
         it 'uses the freebsd ntp servers by default' do
@@ -611,7 +789,8 @@ describe 'ntp' do
 
       describe "on osfamily ArchLinux" do
         let :facts do
-          super().merge({ :osfamily => 'ArchLinux' })
+          super().merge({ :osfamily        => 'ArchLinux',
+                          :operatingsystem => 'ArchLinux'})
         end
 
         it 'uses the ArchLinux NTP servers by default' do
@@ -621,9 +800,11 @@ describe 'ntp' do
         end
       end
 
-      describe "on osfamily Solaris and operatingsystemrelease 5.10" do
+      describe "on osfamily Solaris and kernelrelease 5.10" do
         let :facts do
-          super().merge({ :osfamily => 'Solaris', :operatingsystemrelease => '5.10' })
+          super().merge({ :osfamily        => 'Solaris', 
+                          :kernelrelease   => '5.10',
+                          :operatingsystem => 'Solaris' })
         end
 
         it 'uses the NTP pool servers by default' do
@@ -633,9 +814,11 @@ describe 'ntp' do
         end
       end
 
-      describe "on osfamily Solaris and operatingsystemrelease 5.11" do
+      describe "on osfamily Solaris and kernelrelease 5.11" do
         let :facts do
-          super().merge({ :osfamily => 'Solaris', :operatingsystemrelease => '5.11' })
+          super().merge({ :osfamily        => 'Solaris',
+                          :kernelrelease   => '5.11',
+                          :operatingsystem => 'Solaris' })
         end
 
         it 'uses the NTP pool servers by default' do
@@ -661,7 +844,8 @@ describe 'ntp' do
     describe 'for virtual machines' do
       let :facts do
         super().merge({ :osfamily        => 'Archlinux',
-                     :is_virtual      => 'true' })
+                        :is_virtual      => 'true',
+                        :operatingsystem => 'Archlinux' })
       end
 
       it 'should not use local clock as a time source' do
@@ -680,7 +864,8 @@ describe 'ntp' do
     describe 'for physical machines' do
       let :facts do
         super().merge({ :osfamily        => 'Archlinux',
-                     :is_virtual      => 'false' })
+                        :is_virtual      => 'false',
+                        :operatingsystem => 'Archlinux' })
       end
 
       it 'disallows large clock skews' do
